@@ -1,6 +1,9 @@
+use std::rc::Rc;
+
 use serde::{de::DeserializeOwned, Serialize};
 
 use crate::{
+    authenticate::Authenticator,
     throttle::{Throttle, ThrottleMechanism},
     Error, RestRequest,
 };
@@ -43,6 +46,7 @@ pub struct RestConfig {
     endpoint: String,
     client: reqwest::Client,
     throttle: Option<Throttle>,
+    authenticator: Option<Rc<Box<dyn Authenticator>>>,
 }
 
 impl RestConfig {
@@ -61,6 +65,7 @@ impl RestConfig {
             endpoint: endpoint.as_ref().to_string(),
             client: reqwest::Client::new(),
             throttle: None,
+            authenticator: None,
         }
     }
 
@@ -131,6 +136,19 @@ impl RestConfig {
             request
         };
 
+        let request = if req.authenticate {
+            if let Some(authenticator) = &self.authenticator {
+                authenticator.authenticate(request)
+            } else {
+                log::warn!(
+                    "A request wanted to be authenticated, but no authenticator was registered."
+                );
+                request
+            }
+        } else {
+            request
+        };
+
         // Sending request.
         log::trace!("Sending REST-Request.");
         let response = request.send().await.map_err(|e| Error::Request(e))?;
@@ -141,14 +159,12 @@ impl RestConfig {
         let maybe_json = serde_json::from_slice(&bytes);
         if let Ok(converted) = maybe_json {
             Ok(converted)
+        } else if let Ok(error) = serde_json::from_slice(&bytes) {
+            Err(Error::Api(error))
         } else {
-            if let Ok(error) = serde_json::from_slice(&bytes) {
-                Err(Error::Api(error))
-            } else {
-                Err(Error::BodyNotParsable(
-                    maybe_json.err().expect("Error to be present"),
-                ))
-            }
+            Err(Error::BodyNotParsable(
+                maybe_json.err().expect("Error to be present"),
+            ))
         }
     }
 }
@@ -181,6 +197,12 @@ impl RestConfigBuilder {
         self
     }
 
+    /// Apply a [Authenticator] to authenticate requests.
+    pub fn authenticator<T: 'static + Authenticator>(mut self, authenticator: T) -> Self {
+        self.config.authenticator = Some(Rc::new(Box::new(authenticator)));
+        self
+    }
+
     /// Build the [RestConfig] from the [RestConfigBuilder].
     pub fn build(self) -> RestConfig {
         self.config
